// 3DView.cpp : implementation file
//

#include "stdafx.h"
#include "3DView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#include "MainFrm.h"
#include "CalcEffectDlg.h"

#define MAX_SCALE	30

/////////////////////////////////////////////////////////////////////////////
// C3DView

IMPLEMENT_DYNCREATE(C3DView, CScrollView)

C3DView::C3DView()
{
	m_Mode = 0;
	m_Scale = MAX_SCALE;
	m_Pressed = false;
	m_Moving = false;
	selected = NULL;
}

C3DView::~C3DView()
{
}


BEGIN_MESSAGE_MAP(C3DView, CScrollView)
	//{{AFX_MSG_MAP(C3DView)
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEMOVE()
	ON_COMMAND(ID_DRAWRECT, OnDrawrect)
	ON_UPDATE_COMMAND_UI(ID_DRAWRECT, OnUpdateDrawrect)
	ON_COMMAND(ID_MOVEOBJ, OnMoveobj)
	ON_UPDATE_COMMAND_UI(ID_MOVEOBJ, OnUpdateMoveobj)
	ON_UPDATE_COMMAND_UI(ID_SELECTOBJ, OnUpdateSelectobj)
	ON_COMMAND(ID_SELECTOBJ, OnSelectobj)
	ON_UPDATE_COMMAND_UI(ID_DELOBJ, OnUpdateDelobj)
	ON_COMMAND(ID_DELOBJ, OnDelobj)
	ON_UPDATE_COMMAND_UI(ID_ADDSOUND, OnUpdateAddsound)
	ON_COMMAND(ID_ADDSOUND, OnAddsound)
	ON_WM_KEYDOWN()
	ON_WM_RBUTTONDOWN()
	ON_UPDATE_COMMAND_UI(ID_OBJPROP, OnUpdateObjprop)
	ON_COMMAND(ID_OBJPROP, OnObjprop)
	ON_UPDATE_COMMAND_UI(ID_ZOOMIN, OnUpdateZoomin)
	ON_COMMAND(ID_ZOOMIN, OnZoomin)
	ON_UPDATE_COMMAND_UI(ID_ZOOMOUT, OnUpdateZoomout)
	ON_COMMAND(ID_ZOOMOUT, OnZoomout)
	ON_WM_CREATE()
	ON_COMMAND(ID_CHECKSCENE, OnCheckscene)
	ON_COMMAND(ID_CALCEFFECT, OnCalceffect)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// C3DView drawing

void C3DView::OnDraw(CDC* pDC)
{
	int i;
	if (scene == NULL) return;
	CPen pen(PS_DOT, 1, 0x00A0A0A0);
	pDC->SelectObject(pen);
	int n, m, scale = 1;

	for (i = 0; i < 10; i++)
	{
		n = scale * m_Scale;
		if (n >= 40 && n <= 100) break;
		scale <<= 1;
	}

	n = scene->GetWidth() / scale;
	m = scene->GetHeight() / scale;
	scale *= m_Scale;
	for (i = 1; i <= n; i++)
	{
		pDC->MoveTo(i * scale, 0);
		pDC->LineTo(i * scale, scene->GetHeight() * m_Scale);
	}

	for (i = 1; i <= m; i++)
	{
		pDC->MoveTo(0, i * scale);
		pDC->LineTo(scene->GetWidth() * m_Scale, i * scale);
	}

	scene->DrawScene(pDC, m_Scale);

	pDC->SelectStockObject(BLACK_PEN);
	pDC->MoveTo(0, 0);
	pDC->LineTo(scene->GetWidth() * m_Scale, 0);
	pDC->LineTo(scene->GetWidth() * m_Scale, scene->GetHeight() * m_Scale);
	pDC->LineTo(0, scene->GetHeight() * m_Scale);
	pDC->LineTo(0, 0);
}

/////////////////////////////////////////////////////////////////////////////
// C3DView diagnostics

#ifdef _DEBUG
void C3DView::AssertValid() const
{
	CView::AssertValid();
}

void C3DView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

C3DDoc* C3DView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(C3DDoc)));
	return (C3DDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// C3DView message handlers


void C3DView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	CSceneObject *old;
	CDC *dc = GetDC();
	m_Pressed = true;
	SetCapture();
	double x, y;
	if (!XYToSceneCoord(point.x, point.y, x, y)) return;
	
	switch (m_Mode)
	{
		case 0:
			old = selected;
			selected = scene->SelectObject(selected, x, y, m_Scale);
			if (old != selected) 
			{
				if (old) old->Draw(dc, m_Scale);
				if (selected) selected->Draw(dc, m_Scale);
			}
			break;
		case 1:
			old = selected;
			selected = scene->SelectObject(selected, x, y, m_Scale);
			if (selected)
				if (selected->PtInObj(x, y, m_Scale))
				{
					m_Moving = true;
					old_x = point.x;
					old_y = point.y;
				}
			if (old != selected) 
			{
				if (old) old->Draw(dc, m_Scale);
				if (selected) selected->Draw(dc, m_Scale);
			}
			break;
		case 2:
			old_x = point.x;
			old_y = point.y;
			if (selected) selected->Deselect();
			selected = new CRectangle(x, y, scene->GetParams());
			selected->Select();
			scene->AddObject(selected);
			break;
		case 3:
			if (selected) 
			{
				selected->Deselect();
				selected->Draw(dc, m_Scale);
			}
			selected = new CSound(x, y, scene->GetParams());
			selected->Select();
			scene->AddObject(selected);
			selected->Draw(dc, m_Scale);
			GetDocument()->SetModifiedFlag();
	}
	ReleaseDC(dc);
	CView::OnLButtonDown(nFlags, point);
}


void C3DView::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint) 
{
	// TODO: Add your specialized code here and/or call the base class
	if (scene != GetDocument()->scene) selected = NULL;
	scene = GetDocument()->scene;
	UpdateScale();
}

void C3DView::OnLButtonUp(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	CRectangle *r;
	m_Pressed = false;
	switch (m_Mode)
	{
		case 1: 
			if (selected)
			{
				GetDocument()->SetModifiedFlag();
				selected->EndMove();
				if (old_x != point.x || old_y != point.y) 
					InvalidateRect(NULL);
			}
			break;
		case 2:
			if (selected) 
			{
				r = (CRectangle *)selected;
				if (!r->GetWidth() || !r->GetHeight())
				{
					scene->DeleteObject(r);
					selected = NULL;
					InvalidateRect(NULL);
				}
				else
				{
					selected->EndMove();
					GetDocument()->SetModifiedFlag();
				}
			}
	}
	m_Moving = false;
	ReleaseCapture();
	CScrollView::OnLButtonUp(nFlags, point);
}

void C3DView::OnMouseMove(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	CRect r;
	double x, y, x2, y2;
	CMainFrame *wnd = (CMainFrame *)GetParent();
	if (XYToSceneCoord(point.x, point.y, x, y))
	{
		char s[32];
		sprintf(s, "x = %02.01f, y = %02.01f", x, y);
		wnd->m_wndStatusBar.SetPaneText(1, s);
	}
	else
		wnd->m_wndStatusBar.SetPaneText(1, "");

	if (!m_Pressed) return;
	XYToSceneCoord(old_x, old_y, x2, y2);

	switch (m_Mode)
	{
		case 1:
			if (selected && m_Moving)
			{
				selected->MoveBy(x - x2, y - y2);
				old_x = point.x;
				old_y = point.y;
				InvalidateRect(NULL);
			}
			break;
		case 2:
			((CRectangle *)selected)->SetPoints(x2, y2, x, y);
			InvalidateRect(NULL);
	}

	CScrollView::OnMouseMove(nFlags, point);
}

bool C3DView::XYToSceneCoord(int x, int y, double &x2, double &y2)
{
	C3DDoc *pDoc = GetDocument();
	x2 = x + GetScrollPos(SB_HORZ);
	y2 = y + GetScrollPos(SB_VERT);
	x2 /= m_Scale;
	y2 /= m_Scale;
	if (x2 > scene->GetWidth() || y2 > scene->GetHeight()) return false;
	return true;
}

bool C3DView::SceneToXYCoord(double x, double y, int &x2, int &y2)
{
	CRect r;
	GetClientRect(&r);
	x2 = int(x * m_Scale);
	y2 = int(y * m_Scale);
	x2 -= GetScrollPos(SB_HORZ);
	y2 -= GetScrollPos(SB_VERT);
	if (x2 < 0 || x2 > r.Width() || 
		y2 < 0 || y2 > r.Height()) return false;
	return true;
}

void C3DView::UpdateScrollers()
{
	CSize sizeTotal(scene->GetWidth() * m_Scale, scene->GetHeight() * m_Scale);
	CSize sizePage(sizeTotal.cx >> 3, sizeTotal.cy >> 3);
	CSize sizeLine(m_Scale >> 2, m_Scale >> 2);
	SetScrollSizes(MM_TEXT, sizeTotal, sizePage, sizeLine);
	SetScrollPos(SB_HORZ, 0, FALSE);
	SetScrollPos(SB_VERT, 0, TRUE);
}


void C3DView::OnDrawrect() 
{
	// TODO: Add your command handler code here
	m_Mode = 2;
}

void C3DView::OnUpdateDrawrect(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	pCmdUI->SetCheck(m_Mode == 2);
}

void C3DView::OnMoveobj() 
{
	// TODO: Add your command handler code here
	m_Mode = 1;	
}

void C3DView::OnUpdateMoveobj(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	pCmdUI->SetCheck(m_Mode == 1);	
}

void C3DView::OnUpdateSelectobj(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	pCmdUI->SetCheck(m_Mode == 0);		
}

void C3DView::OnSelectobj() 
{
	// TODO: Add your command handler code here
	m_Mode = 0;		
}

void C3DView::OnUpdateDelobj(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	bool enable = false;
	if (selected)
		if (selected->GetType() != TYPE_LISTENER)
			enable = true;
	pCmdUI->Enable(enable);
}

void C3DView::OnDelobj() 
{
	// TODO: Add your command handler code here
	if (selected)
		if (selected->GetType() != TYPE_LISTENER)
			if (MessageBox("  ?", "", MB_YESNO | MB_ICONEXCLAMATION) == IDYES)
			{
				scene->DeleteObject(selected);
				InvalidateRect(NULL);
				selected = NULL;
				GetDocument()->SetModifiedFlag();
			}
}

void C3DView::OnUpdateAddsound(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	pCmdUI->SetCheck(m_Mode == 3);	
}

void C3DView::OnAddsound() 
{
	// TODO: Add your command handler code here
	m_Mode = 3;
}

void C3DView::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	// TODO: Add your message handler code here and/or call default
	CSceneObject *old;
	CDC *dc = GetDC();
	switch (nChar)
	{
		case VK_UP:
			OnVScroll(SB_LINEUP, 0, NULL);
			break;
		case VK_DOWN:
			OnVScroll(SB_LINEDOWN, 0, NULL);
			break;
		case VK_LEFT:
			OnHScroll(SB_LINELEFT, 0, NULL);
			break;
		case VK_RIGHT:
			OnHScroll(SB_LINERIGHT, 0, NULL);
			break;
		case 0x09:
			old = selected;
			selected = scene->SelectNext(selected);
			if (old != selected) 
			{
				if (old) old->Draw(dc, m_Scale);
				if (selected) selected->Draw(dc, m_Scale);
			}
			break;
	}
	ReleaseDC(dc);
	
	CScrollView::OnKeyDown(nChar, nRepCnt, nFlags);
}


void C3DView::OnRButtonDown(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	CMenu menu;
	menu.LoadMenu(IDR_MAINFRAME);
	CMenu *popup = menu.GetSubMenu(3);
	ClientToScreen(&point);
	popup->TrackPopupMenu(TPM_LEFTALIGN | TPM_RIGHTBUTTON,
		point.x, point.y, GetParent());
	CScrollView::OnRButtonDown(nFlags, point);
}

void C3DView::OnUpdateObjprop(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	pCmdUI->Enable(selected != NULL);	
}

void C3DView::OnObjprop() 
{
	// TODO: Add your command handler code here
	if (selected) selected->PropertyDlg(this);	
}

void C3DView::OnUpdateZoomin(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	pCmdUI->Enable(m_Scale < MAX_SCALE);	
}

void C3DView::OnZoomin() 
{
	// TODO: Add your command handler code here
	m_Scale++;	
	UpdateScale();
}

void C3DView::OnUpdateZoomout(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	pCmdUI->Enable(m_Scale > 1);
}

void C3DView::OnZoomout() 
{
	// TODO: Add your command handler code here
	m_Scale--;
	UpdateScale();
}

void C3DView::UpdateScale()
{
	CMainFrame *wnd = (CMainFrame *)AfxGetMainWnd();
	char s[20];
	sprintf(s, " 1:%.04g", 30.0 / m_Scale);
	if (wnd) wnd->m_wndStatusBar.SetPaneText(2, s);
	UpdateScrollers();
	InvalidateRect(NULL);
}

CDC *C3DView::GetDC()
{
	CDC *dc = CWnd::GetDC();
	dc->SetMapMode(MM_TEXT);
	dc->SetViewportOrg(-GetScrollPos(SB_HORZ), -GetScrollPos(SB_VERT));
	return dc;
}

int C3DView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CScrollView::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	// TODO: Add your specialized creation code here
	SetClassLong(m_hWnd, GCL_HBRBACKGROUND, (long)GetStockObject(WHITE_BRUSH));
	
	return 0;
}

bool C3DView::CheckScene(bool MsgIfOk)
{
	char msg[100];
	if (!scene->CheckScene(msg))
	{
		MessageBox(msg, "", MB_OK | MB_ICONEXCLAMATION);
		return false;
	}
	else if (MsgIfOk)
		MessageBox("   ");
	return true;
}

void C3DView::OnCheckscene() 
{
	// TODO: Add your command handler code here
	CheckScene(true);
}

void C3DView::OnCalceffect() 
{
	if (!CheckScene(false)) return;
	CCalcEffectDlg dlg;		
	dlg.m_Scene = scene;
	dlg.DoModal();
}
