// ConvolDlg.cpp : implementation file
//

#include "stdafx.h"
#include "dsp.h"
#include "ConvolDlg.h"

#include "expr.h"
#include "filters.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define EXPRERROR	char s[100]; \
					e->GetMessage(s); \
					MessageBox(s, "  ", MB_OK | MB_ICONSTOP)

#define WHITERECT(X)	X->SelectStockObject(WHITE_BRUSH); \
						X->SelectStockObject(WHITE_PEN); \
						X->Rectangle(0, 0, G_WIDTH, G_HEIGHT)

#define G_WIDTH		184
#define G_HEIGHT	119

/////////////////////////////////////////////////////////////////////////////
// CConvolDlg dialog


CConvolDlg::CConvolDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CConvolDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CConvolDlg)
	m_ApplyTo = -1;
	m_Function = _T("x");
	m_KernelSize = 512;
	m_Normalize = TRUE;
	//}}AFX_DATA_INIT
}


void CConvolDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CConvolDlg)
	DDX_CBIndex(pDX, IDC_APPLYTO, m_ApplyTo);
	DDX_Text(pDX, IDC_FUNCTION, m_Function);
	DDV_MaxChars(pDX, m_Function, 100);
	DDX_Text(pDX, IDC_KERNELSIZE, m_KernelSize);
	DDV_MinMaxInt(pDX, m_KernelSize, 16, 1024);
	DDX_Check(pDX, IDC_NORMALIZE, m_Normalize);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CConvolDlg, CDialog)
	//{{AFX_MSG_MAP(CConvolDlg)
	ON_BN_CLICKED(IDC_Preview, OnPreview)
	ON_WM_DESTROY()
	ON_WM_PAINT()
	ON_BN_CLICKED(IDAPPLY, OnApply)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CConvolDlg message handlers

void CConvolDlg::OnPreview() 
{
	// TODO: Add your control notification handler code here
	UpdateData();
	try
	{
		BuildKernel();
	}
	catch (ExprException *e)
	{
		EXPRERROR;
		return;
	}
	BuildGraphs();
	InvalidateRect(NULL, FALSE);
}

void CConvolDlg::BuildKernel()
{
	StringExpr *se;
	double x, d, c;
	int i;

	min = 1.7E+308;
	max = -1.7E+308;
	if (!(m_KernelSize & 1)) m_KernelSize--;

	try
	{
		se = new StringExpr(m_Function.GetBuffer(99));
	}
	catch (ExprException *e)
	{
		EXPRERROR;
		return;
	}
	c = 1.0 / (m_KernelSize >> 1);
	for (i = 0; i < m_KernelSize; i++)
	{
		x = -1.0 + i * c;
		d = se->GetValue(x);
		kernel[i] = d;
	}
	if (m_Normalize)
	{
		d = 0.0;
		for (i = 0; i < m_KernelSize; i++)
			d += kernel[i];
		if (fabs(d) < 0.0001) d = 1.0;
		if (d)
			for (i = 0; i < m_KernelSize; i++)
				kernel[i] /= d;
	}
	for (i = 0; i < m_KernelSize; i++)
	{
		d = kernel[i];
		if (d < min) min = d;
		if (d > max) max = d;
	}
}

BOOL CConvolDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	// TODO: Add extra initialization here
	CDC *dc = GetDC();
	bm1 = new CBitmap();
	bm2 = new CBitmap();
	ir = new CDC();
	fr = new CDC();
	bm1->CreateCompatibleBitmap(dc, G_WIDTH, G_HEIGHT);
	bm2->CreateCompatibleBitmap(dc, G_WIDTH, G_HEIGHT);
	ir->CreateCompatibleDC(dc);
	ir->SelectObject(bm1);
	WHITERECT(ir);
	fr->CreateCompatibleDC(dc);
	fr->SelectObject(bm2);
	WHITERECT(fr);
	ReleaseDC(dc);
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CConvolDlg::OnDestroy() 
{
	CDialog::OnDestroy();
	
	// TODO: Add your message handler code here
	delete fr;
	delete ir;
	delete bm2;
	delete bm1;
}

void CConvolDlg::BuildGraphs()
{
	double s1 = 1 / (max - min) * G_HEIGHT, d;
	double s2 = (double)m_KernelSize / G_WIDTH;
	int y, i;
	if (min == max) s1 = 1.0;
	char s[20];

	sprintf(s, "%.2g", max);
	SetDlgItemText(IDC_MAX1, s);
	sprintf(s, "%.2g", min);
	SetDlgItemText(IDC_MIN1, s);

	WHITERECT(ir);
	WHITERECT(fr);
	ir->SelectStockObject(BLACK_PEN);
	fr->SelectStockObject(BLACK_PEN);

	for (i = 0; i < G_WIDTH; i++)
	{
		d = kernel[int(i * s2)];
		y = G_HEIGHT - 1 - int((d - min) * s1);
		if (!i)
			ir->MoveTo(i, y);
		else
			ir->LineTo(i, y);
	}

	int size = 1 << (int)ceil(log(m_KernelSize) / log(2));
	double *re = new double[size];
	double *im = new double[size];

	for (i = m_KernelSize; i < size; i++)
		re[i] = 0.0;
	memcpy(re, kernel, m_KernelSize * sizeof(double));
	ForwardFFT(size, re, im);

	max = -1.7E+308;
	size >>= 1;
	for (i = 0; i < size; i++)
	{
		re[i] = sqrt(re[i] * re[i] + im[i] * im[i]);
		if (re[i] < 0.0001) re[i] = 0.0001;
		re[i] = 20 * log(re[i]);
		if (re[i] > max) max = re[i];
	}

	if (max > 0 && max < 0.01) max = 0.0;
	sprintf(s, "%.2g", max);
	SetDlgItemText(IDC_MAX2, s);
	s1 = 1 / (max + 80.0) * G_HEIGHT;
	s2 = (double)size / G_WIDTH;

	for (i = 0; i < G_WIDTH; i++)
	{
		d = re[int(i * s2)] + 80.0;
		y = G_HEIGHT - 1 - int(d * s1);
		if (!i)
			fr->MoveTo(i, y);
		else
			fr->LineTo(i, y);
	}

	delete im;
	delete re;
}


void CConvolDlg::OnPaint() 
{
	CPaintDC dc(this); // device context for painting
	
	// TODO: Add your message handler code here
	
	dc.BitBlt(25, 43, G_WIDTH, G_HEIGHT, ir, 0, 0, SRCCOPY);
	dc.BitBlt(257, 43, G_WIDTH, G_HEIGHT, fr, 0, 0, SRCCOPY);
	// Do not call CDialog::OnPaint() for painting messages
}

void CConvolDlg::OnApply() 
{
	// TODO: Add your control notification handler code here
	UpdateData();
	try
	{
		BuildKernel();
	}
	catch (ExprException *e)
	{
		EXPRERROR;
		return;
	}
	OnOK();
}
