// DSPView.cpp : implementation of the CDSPView class
//

#include "stdafx.h"
#include "DSP.h"

#include "DSPDoc.h"
#include "DSPView.h"
#include "MainFrm.h"
#include <stdlib.h>
#include <string.h>
#include "filters.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CDSPView

IMPLEMENT_DYNCREATE(CDSPView, CView)

BEGIN_MESSAGE_MAP(CDSPView, CView)
	//{{AFX_MSG_MAP(CDSPView)
	ON_COMMAND(ID_LINEAR, OnLinear)
	ON_UPDATE_COMMAND_UI(ID_LINEAR, OnUpdateLinear)
	ON_COMMAND(ID_LOGARITHMIC, OnLogarithmic)
	ON_UPDATE_COMMAND_UI(ID_LOGARITHMIC, OnUpdateLogarithmic)
	ON_COMMAND(ID_ZOOMIN, OnZoomin)
	ON_COMMAND(ID_ZOOMOUT, OnZoomout)
	ON_COMMAND(ID_SAMPLES, OnSamples)
	ON_UPDATE_COMMAND_UI(ID_SAMPLES, OnUpdateSamples)
	ON_COMMAND(ID_SPECTROGRAM, OnSpectrogram)
	ON_UPDATE_COMMAND_UI(ID_SPECTROGRAM, OnUpdateSpectrogram)
	ON_COMMAND(ID_HISTOGRAM, OnHistogram)
	ON_UPDATE_COMMAND_UI(ID_HISTOGRAM, OnUpdateHistogram)
	ON_UPDATE_COMMAND_UI(ID_ZOOMIN, OnUpdateZoomin)
	ON_UPDATE_COMMAND_UI(ID_ZOOMOUT, OnUpdateZoomout)
	ON_WM_SIZE()
	ON_WM_HSCROLL()
	ON_WM_ERASEBKGND()
	ON_WM_VSCROLL()
	ON_WM_KEYDOWN()
	ON_WM_KEYUP()
	ON_WM_SETCURSOR()
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEMOVE()
	ON_WM_TIMER()
	ON_UPDATE_COMMAND_UI(ID_EDIT_COPY, OnUpdateEditCopy)
	ON_UPDATE_COMMAND_UI(ID_EDIT_CUT, OnUpdateEditCut)
	ON_UPDATE_COMMAND_UI(ID_EDIT_CLEAR, OnUpdateEditClear)
	ON_UPDATE_COMMAND_UI(ID_EDIT_PASTE, OnUpdateEditPaste)
	ON_COMMAND(ID_GOTOEND, OnGotoend)
	ON_COMMAND(ID_GOTOSTART, OnGotostart)
	ON_COMMAND(ID_FORWARD, OnForward)
	ON_COMMAND(ID_REWIND, OnRewind)
	ON_UPDATE_COMMAND_UI(ID_CLEAR, OnUpdateClear)
	ON_COMMAND(ID_SELECTALL, OnSelectall)
	ON_COMMAND(ID_SELFRAGMENT, OnSelfragment)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDSPView construction/destruction

CDSPView::CDSPView()
{
	// TODO: add construction code here
	m_IsLinear = TRUE;
	m_OldScale = m_Scale = 4;
	m_Form = 0;
	m_Control = FALSE;
	m_StartPos = m_EndPos = 0;
	m_Pressed = FALSE;
	m_IncTime = 200;
	m_ScaleFnt.CreateFont(14, 0, 0, 0, FW_DONTCARE, FALSE, FALSE, FALSE,
		DEFAULT_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, 
		DEFAULT_QUALITY, FF_DONTCARE, "MS Serif");
	m_DarkPen.CreatePen(PS_SOLID, 1, GetSysColor(COLOR_BTNSHADOW));
	m_FacePen.CreatePen(PS_SOLID, 1, GetSysColor(COLOR_BTNFACE));
	m_LightPen.CreatePen(PS_SOLID, 1, GetSysColor(COLOR_BTNHIGHLIGHT));
	m_BluePen.CreatePen(PS_SOLID, 1, RGB(0, 0, 0x80));
	m_FaceBrush.CreateSolidBrush(GetSysColor(COLOR_BTNFACE));
	for (int i = 0; i < 256; i++)
		m_GrayPen[i].CreatePen(PS_SOLID, 0, RGB(i, i, i));
}

CDSPView::~CDSPView()
{
	DestroyCaret();
}

BOOL CDSPView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CDSPView drawing

void CDSPView::OnDraw(CDC* pDC)
{
	CDSPDoc *pDoc = GetDocument();
	CBitmap BM;
	CRect r;
	CDC DC;
	int n = GetScrollPos(SB_VERT);
	int y1 = -n, y2 = -n, y3 = C_HEIGHT + 1 - n;

	ASSERT_VALID(pDoc);
	// TODO: add draw code for native data here

	CheckPosition();

	GetClientRect(&r);
	if (r.Height() > C_HEIGHT) 
		y1 = (r.Height() - C_HEIGHT) >> 1;
	if (r.Height() > C_HEIGHT * 2)
	{
		y2 = ((r.Height() >> 1) - C_HEIGHT) >> 1;
		y3 = C_HEIGHT + 3 * y2;
	}

	DC.CreateCompatibleDC(pDC);
	BM.CreateCompatibleBitmap(pDC, r.Width(), r.Height());
	DC.SelectObject(BM);
	DC.SelectStockObject(WHITE_PEN);
	DC.SelectStockObject(WHITE_BRUSH);
	DC.Rectangle(&r);
	DC.SetBkMode(TRANSPARENT);

	int N = min(r.Width(), m_nSamples >> m_Scale);
	int Is16bit = pDoc->h.wBitsPerSample >> 4;

	if (pDoc->data != NULL)
	if (m_nChannels > 1)
	{
		char str[20];
		switch (m_Form)
		{
			case 0:  
				DrawChannel_0(&DC, r, Is16bit, 0, N, y2, pDoc->data);
				DrawChannel_0(&DC, r, Is16bit, 1, N, y3, pDoc->data);
				break;
			case 1: 
				DrawChannel_1(&DC, r, Is16bit, 0, N, y2, pDoc->data);
				DrawChannel_1(&DC, r, Is16bit, 1, N, y3, pDoc->data);
				break;
			case 2: 
				DrawChannel_2(&DC, r, Is16bit, 0, N, y2, pDoc->data);
				DrawChannel_2(&DC, r, Is16bit, 1, N, y3, pDoc->data);
		}
		DC.SelectObject(m_DarkPen);
		DC.MoveTo(0, 2 * y2 + C_HEIGHT + n);
		DC.LineTo(r.right, 2 * y2 + C_HEIGHT + n);
		strcpy(str, " ");
		DC.TextOut(10, 10 + y2, str);
		strcpy(str, " ");
		DC.TextOut(10, 10 + y3, str);
	}
	else
		switch (m_Form)
		{
			case 0: DrawChannel_0(&DC, r, Is16bit, 0, N, y1, pDoc->data); break;
			case 1: DrawChannel_1(&DC, r, Is16bit, 0, N, y1, pDoc->data); break;
			case 2: DrawChannel_2(&DC, r, Is16bit, 0, N, y1, pDoc->data); 
		}

	if ((m_nSamples >> m_Scale) < r.Width())
	{
		DC.SelectObject(m_DarkPen);
		DC.MoveTo((m_nSamples >> m_Scale) + 1, 0);
		DC.LineTo((m_nSamples >> m_Scale) + 1, r.Height());
	}

	if (m_Form != 2) 
		UpdateCaret(&DC);
	else
		HideCaret();

	if (pDoc->IsPlaying && !m_Form)
	{
		int x = (m_PlayPos - m_Pos) >> m_Scale;
		DC.InvertRect(CRect(x, 0, x + 1, r.bottom));
	}

	pDC->BitBlt(S_WIDTH, S_HEIGHT, r.Width(), r.Height(), &DC, 0, 0, SRCCOPY);
	pDC->SelectObject(m_FaceBrush);
	pDC->SelectObject(m_FacePen);
	pDC->Rectangle(0, 0, S_WIDTH - 2, S_HEIGHT - 2);
	pDC->SelectStockObject(BLACK_PEN);
	pDC->MoveTo(0, S_HEIGHT - 1);
	pDC->LineTo(S_WIDTH - 1, S_HEIGHT - 1);
	pDC->LineTo(S_WIDTH - 1, -1);
	pDC->SelectObject(m_DarkPen);
	pDC->MoveTo(0, S_HEIGHT - 2);
	pDC->LineTo(S_WIDTH - 2, S_HEIGHT - 2);
	pDC->LineTo(S_WIDTH - 2, -1);
	DrawHorzScale(pDC);
	DrawVertScale(pDC, y1, y2, y3);

	CMainFrame *pFrame = (CMainFrame *)AfxGetApp()->m_pMainWnd;
	pStatus = &pFrame->m_wndStatusBar;

	UpdateScale();
	UpdatePosition();
	UpdateParameters(pDoc);
}

void CDSPView::DrawHorzScale(CDC *pDC)
{
	CDSPDoc *pDoc = GetDocument();
	CBitmap BM;
	CDC DC;
	CRect r;
	char str[20];
	int pos, height, x;
	double x_inc, x2;

	CView::GetClientRect(&r);
	DC.CreateCompatibleDC(pDC);
	BM.CreateCompatibleBitmap(pDC, r.Width() - S_WIDTH, S_HEIGHT);
	DC.SelectObject(BM);
	DC.SelectObject(m_FaceBrush);
	DC.SelectObject(m_FacePen);
	DC.Rectangle(&r);

	DC.SelectStockObject(BLACK_PEN);
	DC.MoveTo(0, S_HEIGHT - 1);
	DC.LineTo(r.right - S_WIDTH, S_HEIGHT - 1);

	DC.SelectObject(m_DarkPen);
	DC.MoveTo(0, S_HEIGHT - 2);
	DC.LineTo(r.right - S_WIDTH, S_HEIGHT - 2);

	DC.SetBkColor(GetSysColor(COLOR_BTNFACE));
	DC.SelectObject(m_ScaleFnt);

	if (m_Form != 2)
	{
		pos = GetSampleTime(m_Pos);
		x_inc = (double)(GetSampleForTime(m_IncTime) >> m_Scale) / 10;
		pos /= m_IncTime;
		if (pos * m_IncTime < GetSampleTime(m_Pos) - m_IncTime) pos++;
		pos *= m_IncTime;
		do
		{
			x = (GetSampleForTime(pos) - m_Pos) >> m_Scale;
			GetTimeString(pos, str);
			DC.TextOut(x + 2, 0, str);
			x2 = (double)x;
			for (int i = 0; i < 10; i++)
			{
				if (!i)
					height = 5;
				else
					height = 3;
				DC.SelectStockObject(BLACK_PEN);
				DC.MoveTo((int)x2, S_HEIGHT - height - 1);
				DC.LineTo((int)x2, S_HEIGHT - 1);
				DC.SelectObject(m_LightPen);
				DC.MoveTo((int)x2 + 1, S_HEIGHT - height - 1);
				DC.LineTo((int)x2 + 1, S_HEIGHT - 1);
				x2 += x_inc;
			}
			pos += m_IncTime;
		} while (x < r.right - S_WIDTH);
	}
	else
	{
		int N = min(r.Width(), m_nSamples >> m_Scale);
		int n = 1 << int(log(N) / log(2));
		int mul[] = {100, 200, 500, 1000, 2000, 5000, 10000}, m;
		double x_inc2, x3;
		x2 = 0;

		for (int i = 0; i < 7; i++)
		{
			m = mul[i];
			x_inc = (double)m / (pDoc->h.w.nSamplesPerSec / n / 2);
			if (x_inc > 50) break;
		}

		x_inc2 = x_inc / 10;
		height = 3;
		i = 0;

		x = int(x2);
		do
		{
			x3 = x2;

			for (int j = 0; j < 10; j++)
			{
				if (!j)
					height = 5;
				else
					height = 3;
				DC.SelectStockObject(BLACK_PEN);
				DC.MoveTo((int)x3, S_HEIGHT - height - 1);
				DC.LineTo((int)x3, S_HEIGHT - 1);
				DC.SelectObject(m_LightPen);
				DC.MoveTo((int)x3 + 1, S_HEIGHT - height - 1);
				DC.LineTo((int)x3 + 1, S_HEIGHT - 1);
				x3 += x_inc2;
			}

			itoa(i * m, str, 10);
			DC.TextOut(x + 2, 0, str);

			x2 += x_inc;
			x = int(x2);
			i++;
		} while (x < n);
	}

	DC.SelectObject(m_LightPen);
	DC.MoveTo(0, 0);
	DC.LineTo(0, S_HEIGHT);

	pDC->BitBlt(S_WIDTH, 0, r.Width(), S_HEIGHT, &DC, 0, 0, SRCCOPY);
}

void CDSPView::DrawVertScale(CDC *pDC, int y1, int y2, int y3)
{
	CDSPDoc *pDoc = GetDocument();
	CBitmap BM;
	CDC DC;
	CRect r;
	int offs;
	char s[10];

	CView::GetClientRect(&r);
	DC.CreateCompatibleDC(pDC);
	BM.CreateCompatibleBitmap(pDC, S_WIDTH, r.Height() - S_HEIGHT);
	DC.SelectObject(BM);
	DC.SelectObject(m_FaceBrush);
	DC.SelectObject(m_FacePen);
	DC.Rectangle(&r);

	DC.SelectStockObject(BLACK_PEN);
	DC.MoveTo(S_WIDTH - 1, 0);
	DC.LineTo(S_WIDTH - 1, r.bottom - S_HEIGHT);

	DC.SelectObject(m_DarkPen);
	DC.MoveTo(S_WIDTH - 2, 0);
	DC.LineTo(S_WIDTH - 2, r.bottom - S_HEIGHT);

	if (m_nChannels > 1)
	{
		offs = (y2 + y3 + C_HEIGHT) / 2 - 1;
		DC.SelectObject(m_DarkPen);
		DC.MoveTo(0, offs);
		DC.LineTo(S_WIDTH, offs);
		DC.SelectObject(m_LightPen);
		DC.MoveTo(0, offs + 2);
		DC.LineTo(S_WIDTH - 1, offs + 2);
		DC.SelectStockObject(BLACK_PEN);
		DC.MoveTo(0, offs + 1);
		DC.LineTo(S_WIDTH, offs + 1);
		offs = y2 - 1;
	}
	else offs = y1 - 1;

	DC.SetBkColor(GetSysColor(COLOR_BTNFACE));

	if (m_Form != 1)
	for (int i = 0; i < m_nChannels; i++)
	{
		double x_inc = C_HEIGHT / (double)TICKS;
		double x = x_inc;
		int width;
		for (int j = 0; j < TICKS - 1; j++, x += x_inc)
		{
			if (!m_Form)
			{
				if (j == TICKS / 2 - 1) width = 5;
				else width = 3;
			}
			else
				width = 3;
			DC.SelectStockObject(BLACK_PEN);
			DC.MoveTo(S_WIDTH - width, (int)x + offs);
			DC.LineTo(S_WIDTH - 1, (int)x + offs);
			DC.SelectObject(m_LightPen);
			DC.MoveTo(S_WIDTH - width, (int)x + offs + 1);
			DC.LineTo(S_WIDTH - 1, (int)x + offs + 1);
			if (m_IsLinear && !m_Form)
			{
				if (pDoc->h.wBitsPerSample == 8)
				{
					if (!j) 
						strcpy(s, "112");
					else if (j == TICKS - 2)
						strcpy(s, "-112");
					else
						itoa((128 - 256 / TICKS) - j * (256 / TICKS), s, 10);
				}
				else
				{
					if (!j) 
						strcpy(s, "28672");
					else if (j == TICKS - 2)
						strcpy(s, "-28672");
					else
						itoa((32768 - 65536 / TICKS) - j * (65536 / TICKS), s, 10);
				}
			}
			else
			{
				if (pDoc->h.wBitsPerSample == 8)
				{
					if (!m_Form)
					{
						if (!j || j == TICKS - 2) 
							strcpy(s, "-5");
						else if (j == TICKS / 2 - 1)
							strcpy(s, "-Inf");
						else
							itoa(int(-42 + fabs(j - TICKS / 2 + 1) * ((double)84.288 / TICKS)), s, 10);
					}
					else
						itoa(int(-(j + 1) * ((double)42.144 / TICKS)), s, 10);
				}
				else
				{
					if (!m_Form)
					{
						if (!j || j == TICKS - 2) 
							strcpy(s, "-11");
						else if (j == TICKS / 2 - 1)
							strcpy(s, "-Inf");
						else
							itoa(int(-90 + fabs(j - TICKS / 2 + 1) * ((double)180.618 / TICKS)), s, 10);
					}
					else
						itoa(int(-(j + 1) * ((double)90.309 / TICKS)), s, 10);
				}
			}
			DC.SelectObject(m_ScaleFnt);
			DC.SetTextAlign(TA_RIGHT);
			DC.TextOut(S_WIDTH - 8, (int)x + offs - 6, s);
		}
		offs = y3;
	}
	else if (m_Form == 1)
	for (int i = 0; i < m_nChannels; i++)
	{
		double x_inc = C_HEIGHT / (double)TICKS;
		double freq_inc = (double)pDoc->h.w.nSamplesPerSec / 2 / TICKS;
		double x = x_inc;
		for (int j = 0; j < TICKS - 1; j++, x += x_inc)
		{
			DC.SelectStockObject(BLACK_PEN);
			DC.MoveTo(S_WIDTH - 3, (int)x + offs);
			DC.LineTo(S_WIDTH - 1, (int)x + offs);
			DC.SelectObject(m_LightPen);
			DC.MoveTo(S_WIDTH - 3, (int)x + offs + 1);
			DC.LineTo(S_WIDTH - 1, (int)x + offs + 1);
			itoa(int((TICKS - 1 - j) * freq_inc), s, 10);
			DC.SelectObject(m_ScaleFnt);
			DC.SetTextAlign(TA_RIGHT);
			DC.TextOut(S_WIDTH - 8, (int)x + offs - 6, s);
		}
		offs = y3;
	}

	DC.SelectObject(m_LightPen);
	DC.MoveTo(0, 0);
	DC.LineTo(S_WIDTH - 1, 0);

	pDC->BitBlt(0, S_HEIGHT, S_WIDTH, r.Height(), &DC, 0, 0, SRCCOPY);
}

int CDSPView::GetSampleForTime(int msecs)
{
	CDSPDoc *pDoc = GetDocument();
	double t = 1.0 / pDoc->h.w.nSamplesPerSec;
	return int((double)msecs / 1000.0 / t);
}

int CDSPView::GetSampleTime(int Sample)
{
	CDSPDoc *pDoc = GetDocument();
	return int((double)Sample * 1000.0 / pDoc->h.w.nSamplesPerSec);
}

void CreateMyCaret(HWND hWnd, int w, int h)
{
	CreateCaret(hWnd, 0, w, h);
}

void CDSPView::UpdateCaret(CDC *pDC)
{
	CRect r;
	int x1 = min(m_StartPos, m_EndPos), x2 = max(m_StartPos, m_EndPos);
	int x3, x4, w; 
	CDSPDoc *pDoc = GetDocument();

	GetClientRect(&r);
	x3 = (x1 - m_Pos) >> m_Scale;
	x4 = (x2 - m_Pos) >> m_Scale; 
	if (x3 < 0) x3 = 0;
	if (x4 > r.right) x4 = r.right;
	if (x4 < x3) 
		w = 0;
	else
		w = x4 - x3 + 1;

	if (!w)
		HideCaret();
	else
	{
		CreateMyCaret(m_hWnd, 1, r.Height());
		if (w > 1)
			pDC->InvertRect(CRect(x3, 0, x3 + w, r.bottom));
		if (x4 - x3 > 0) 
			if (!pDoc->IsPlaying) HideCaret();
		else
			ShowCaret();
		SetCaretPos(CPoint(x3 + S_WIDTH, S_HEIGHT));
	}
}

void CDSPView::DrawChannel_0(CDC *pDC, CRect r, int Is16bit, int IsRight, int N, int Y, char *data)
{
	int s;

	pDC->SelectStockObject(BLACK_PEN);
	pDC->MoveTo(0, 127 + Y);
	pDC->LineTo(r.right, 127 + Y);
	pDC->SelectObject(m_BluePen);

	for (int i = 0; i < N; i++)
	{
		int index = ((i << m_Scale) + m_Pos) << (m_nChannels - 1);
		if (Is16bit)
		{
			s = (unsigned short)(((unsigned short *)data)[index + IsRight] - 32768);
			s >>= 8;
		}
		else
			s = (unsigned char)data[index + IsRight];
		s = 255 - s;
		if (!m_IsLinear)
		{
			double d = s;
			d -= 127;
			d = fabs(d);
			if (d > 0)
			{
				d = 20 * log10(d / 128);
				d = fabs(d * 3.037191401);
			}
			else d = 127;
			if (s < 128) s = (int)d;
			else s = 256 - (int)d;
		}
		if (!i) 
			pDC->MoveTo(0, s + Y);
		else
			pDC->LineTo(i, s + Y);
	}
}

void CDSPView::DrawChannel_1(CDC *pDC, CRect r, int Is16bit, int IsRight, int N, int Y, char *data)
{
	double sp[SG_SIZE >> 1];
	int size = GetSampleSize(), index = m_Pos * size, t, i, j;
	int k = C_HEIGHT >> (SG_LOG - 1);
	CPen Pen;

	for (i = 0; i < N; i++, index += (1 << m_Scale) * size)
	{
		GetSpectrum(SampleData(SG_SIZE, data + index, Is16bit, m_nChannels - 1, IsRight), sp);
		for (j = 0; j < SG_SIZE >> 1; j++)
		{
			if (Is16bit)
				sp[j] = -LevelTo_dB(sp[j], 32767.0, -90.3) * 255.0 / 90.3;
			else
				sp[j] = -LevelTo_dB(sp[j], 127.0, -42) * 255.0 / 42;
			pDC->SelectObject(m_GrayPen[int(sp[j])]);
			t = Y + C_HEIGHT;
			pDC->MoveTo(i, t - j * k);
			pDC->LineTo(i, t - j * k - k);
		}
	}
}

void CDSPView::DrawChannel_2(CDC *pDC, CRect r, int Is16bit, int IsRight, int N, int Y, char *data)
{
	int n = 1 << int(log(N) / log(2));
	int index = m_Pos * GetSampleSize();
	double *sp = new double[n >> 1];

	GetSpectrum(SampleData(n, data + index, Is16bit, m_nChannels - 1, IsRight), sp);

	for (int i = 0; i < n >> 1; i++)
	{	
		if (Is16bit)
			sp[i] = -LevelTo_dB(sp[i], 32767.0, -90.3) * 256.0 / 90.3;
		else
			sp[i] = -LevelTo_dB(sp[i], 127.0, -42) * 256.0 / 42;
		pDC->SelectObject(m_BluePen);
		pDC->MoveTo(i << 1, Y + C_HEIGHT);
		pDC->LineTo(i << 1, Y + int(sp[i]));
		pDC->MoveTo((i << 1) + 1, Y + C_HEIGHT);
		pDC->LineTo((i << 1) + 1, Y + int(sp[i]));
	}
	
	delete sp;
}

/////////////////////////////////////////////////////////////////////////////
// CDSPView diagnostics

#ifdef _DEBUG
void CDSPView::AssertValid() const
{
	CView::AssertValid();
}

void CDSPView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CDSPDoc* CDSPView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CDSPDoc)));
	return (CDSPDoc*)m_pDocument;
}
#endif //_DEBUG


void CDSPView::OnLinear() 
{
	// TODO: Add your command handler code here
	m_IsLinear = TRUE;	
	InvalidateRect(NULL, FALSE);
}

void CDSPView::OnLogarithmic() 
{
	// TODO: Add your command handler code here
	m_IsLinear = FALSE;
	InvalidateRect(NULL, FALSE);
}

void CDSPView::OnUpdateLinear(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	pCmdUI->Enable(!m_Form);
	pCmdUI->SetCheck(m_IsLinear && !m_Form);	
}

void CDSPView::OnUpdateLogarithmic(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	pCmdUI->Enable(!m_Form);
	pCmdUI->SetCheck(!m_IsLinear && !m_Form);	
}

void CDSPView::OnZoomin() 
{
	// TODO: Add your command handler code here
	m_Scale--;
	m_OldScale = m_Scale;
	UpdateScrollParams();
	CalcIncTime();
	InvalidateRect(NULL, FALSE);
}

void CDSPView::OnZoomout() 
{
	// TODO: Add your command handler code here
	m_Scale++;	
	m_OldScale = m_Scale;
	UpdateScrollParams();
	CalcIncTime();
	InvalidateRect(NULL, FALSE);
}

void CDSPView::UpdateScale()
{
	int n = 1 << m_Scale;
	char str[6] = "1:", str2[3];

	itoa(n, str2, 10);
	strcat(str, str2);
	if (!m_Form)
		pStatus->SetPaneText(1, str);
	else
		pStatus->SetPaneText(1, "");
}

void CDSPView::CheckPosition()
{
	if (m_StartPos > m_nSamples - 1) m_StartPos = m_nSamples - 1;
	if (m_StartPos < 0) m_StartPos = 0;
	if (m_EndPos > m_nSamples - 1) m_EndPos = m_nSamples - 1;
	if (m_EndPos < 0) m_EndPos = 0;

	int s = (m_nSamples >> m_Scale) << m_Scale;
	if (m_Pos > s - m_PageSize) m_Pos = s - m_PageSize;
	if (m_Pos < 0) m_Pos = 0;

	m_Pos = (m_Pos >> m_Scale) << m_Scale;
	m_StartPos = (m_StartPos >> m_Scale) << m_Scale;
	m_EndPos = (m_EndPos >> m_Scale) << m_Scale;
}

void CDSPView::UpdatePosition()
{
	char str[20];

	if (GetDocument()->IsPlaying)
		GetSampleTime(m_PlayPos, str);
	else
	{
		if (m_Form != 2)
			GetSampleTime(GetStartPos(), str);
		else	
			GetSampleTime(m_Pos, str);
	}

	pStatus->SetPaneText(2, str);
	if (m_StartPos != m_EndPos && m_Form != 2 && !GetDocument()->IsPlaying)
	{
		GetSampleTime(GetEndPos(), str);
		pStatus->SetPaneText(3, str);
		GetSampleTime(GetEndPos() - GetStartPos() + 1, str);
		pStatus->SetPaneText(4, str);
	}
	else
	{
		pStatus->SetPaneText(3, "");
		pStatus->SetPaneText(4, "");
	}
}

void CDSPView::UpdateParameters(CDSPDoc *pDoc)
{
	char str[15];

	ltoa(pDoc->h.w.nSamplesPerSec, str, 10);
	strcat(str, " ");
	pStatus->SetPaneText(5, str);
	itoa(pDoc->h.wBitsPerSample, str, 10);
	strcat(str, " ");
	pStatus->SetPaneText(6, str);
	if (m_nChannels == 1)
		pStatus->SetPaneText(7, "");
	else
		pStatus->SetPaneText(7, "");
}

void CDSPView::UpdateScrollParams()
{
	CRect r;
	SCROLLINFO i;
	GetClientRect(&r);
	m_StepSize = 1 << m_Scale;
	m_PageSize = m_StepSize * r.Width();
	i.nPage = int((double)m_PageSize * 32768.0 / m_nSamples);
	i.nMin = 0;
	i.nMax = 32768 - 1;
	i.cbSize = sizeof(SCROLLINFO);
	i.fMask = SIF_PAGE | SIF_RANGE;
	SetScrollInfo(SB_HORZ, &i);
	i.nPage = r.Height();
	i.nMax = 256 * m_nChannels - 1;
	SetScrollInfo(SB_VERT, &i);
	SetScrollPos(SB_HORZ, (int)((double)m_Pos * 32768.0 / m_nSamples));
}

void CDSPView::OnSamples() 
{
	// TODO: Add your command handler code here
	m_Scale = m_OldScale;
	m_Form = 0;
	CalcIncTime();
	UpdateScrollParams();
	InvalidateRect(NULL, FALSE);
}

void CDSPView::OnUpdateSamples(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	pCmdUI->SetCheck((m_Form == 0));
}

void CDSPView::OnSpectrogram() 
{
	// TODO: Add your command handler code here
	m_OldScale = m_Scale;
	m_Form = 1;
	m_Scale = SG_LOG;
	CalcIncTime();
	UpdateScrollParams();
	InvalidateRect(NULL, FALSE);
}

void CDSPView::OnUpdateSpectrogram(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	pCmdUI->SetCheck((m_Form == 1));	
}

void CDSPView::OnHistogram() 
{
	// TODO: Add your command handler code here
	m_Scale = m_OldScale;
	m_Form = 2;
	UpdateScrollParams();
	InvalidateRect(NULL, FALSE);
}

void CDSPView::OnUpdateHistogram(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	pCmdUI->SetCheck((m_Form == 2));	
}

void CDSPView::OnUpdateZoomin(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	pCmdUI->Enable(m_Scale > 0 && !m_Form);
}

void CDSPView::OnUpdateZoomout(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	pCmdUI->Enable(m_Scale < 8 && !m_Form);	
}

void CDSPView::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint) 
{
	// TODO: Add your specialized code here and/or call the base class
	CDSPDoc *pDoc = GetDocument();
	m_nSamples = pDoc->h.size2 / (pDoc->h.w.nChannels * pDoc->h.wBitsPerSample >> 3);
	m_nChannels = pDoc->h.w.nChannels;
	m_Pos = 0;
	m_StartPos = m_EndPos = 0;
	CalcIncTime();
	SetScrollPos(SB_HORZ, 0);
	SetScrollPos(SB_VERT, 0);
	UpdateScrollParams();
	InvalidateRect(NULL, FALSE);
}

void CDSPView::CalcIncTime()
{
	CDSPDoc *pDoc = GetDocument();
	int mul[] = {5, 10, 20, 50, 100, 200, 500, 1000, 2000};

	double t = (double)100.0 * (1 << m_Scale) / (pDoc->h.w.nSamplesPerSec);

	for (int i = 8; i >= 0; i--)
	{
		m_IncTime = int(t * 1000);
		if (m_IncTime % mul[i]) m_IncTime += mul[i];
		m_IncTime = m_IncTime / mul[i] * mul[i];
		if ((GetSampleForTime(m_IncTime) >> m_Scale) < 200) break;
	}
}

void CDSPView::OnSize(UINT nType, int cx, int cy) 
{
	CView::OnSize(nType, cx, cy);
	
	// TODO: Add your message handler code here
	UpdateScrollParams();	
}

void CDSPView::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	// TODO: Add your message handler code here and/or call default
	SCROLLINFO i;

	switch (nSBCode)
	{
		case SB_LINERIGHT: m_Pos += m_StepSize << 4; break;
		case SB_LINELEFT: m_Pos -= m_StepSize << 4; break;
		case SB_PAGERIGHT: m_Pos += m_PageSize; break;
		case SB_PAGELEFT: m_Pos -= m_PageSize; break;
		case SB_THUMBTRACK: m_Pos = (int)((double)nPos * m_nSamples / 32768.0);
			m_Pos = (m_Pos >> m_Scale) << m_Scale;
	}
	
	int s = (m_nSamples >> m_Scale) << m_Scale;
	if (m_Pos > s - m_PageSize || nSBCode == SB_RIGHT)
		m_Pos = s - m_PageSize;
	if (m_Pos < 0 || nSBCode == SB_LEFT) m_Pos = 0;
	
	GetScrollInfo(SB_HORZ, &i);
	if (nSBCode != SB_THUMBTRACK)
		SetScrollPos(SB_HORZ, (int)((double)m_Pos * 32768.0 / m_nSamples));
	InvalidateRect(NULL, FALSE);

	CView::OnHScroll(nSBCode, nPos, pScrollBar);
}

BOOL CDSPView::OnEraseBkgnd(CDC* pDC) 
{
	// TODO: Add your message handler code here and/or call default
	return TRUE;
}

void CDSPView::OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	// TODO: Add your message handler code here and/or call default
	SCROLLINFO i;

	GetScrollInfo(SB_VERT, &i);

	switch (nSBCode)
	{
		case SB_LINERIGHT: i.nPos += 16; break;
		case SB_LINELEFT: i.nPos -= 16; break;
		case SB_PAGERIGHT: i.nPos += i.nPage; break;
		case SB_PAGELEFT: i.nPos -= i.nPage; break;
		case SB_THUMBTRACK: i.nPos = nPos;
	}

	SetScrollPos(SB_VERT, i.nPos);
	InvalidateRect(NULL, FALSE);
	
	CView::OnVScroll(nSBCode, nPos, pScrollBar);
}

void CDSPView::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	// TODO: Add your message handler code here and/or call default
	switch (nChar)
	{
		case VK_HOME:
			OnHScroll(SB_LEFT, 0, NULL);
			break;
		case VK_END:
			OnHScroll(SB_RIGHT, 0, NULL);
			break;
		case VK_UP:
			OnVScroll(SB_LINEUP, 0, NULL);
			break;
		case VK_DOWN:
			OnVScroll(SB_LINEDOWN, 0, NULL);
			break;
		case VK_PRIOR:
			OnVScroll(SB_PAGEUP, 0, NULL);
			break;
		case VK_NEXT:
			OnVScroll(SB_PAGEDOWN, 0, NULL);
			break;
		case VK_LEFT:
			if (m_Control)
				OnHScroll(SB_PAGELEFT, 0, NULL);
			else
				OnHScroll(SB_LINELEFT, 0, NULL);
			break;
		case VK_RIGHT:
			if (m_Control)
				OnHScroll(SB_PAGERIGHT, 0, NULL);
			else
				OnHScroll(SB_LINERIGHT, 0, NULL);
			break;
		case VK_CONTROL:
			m_Control = TRUE;
			break;
	}

	CView::OnKeyDown(nChar, nRepCnt, nFlags);
}

void CDSPView::OnKeyUp(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	// TODO: Add your message handler code here and/or call default
	if (nChar == VK_CONTROL)
		m_Control = FALSE;

	CView::OnKeyUp(nChar, nRepCnt, nFlags);
}

BOOL CDSPView::OnSetCursor(CWnd* pWnd, UINT nHitTest, UINT message) 
{
	// TODO: Add your message handler code here and/or call default
	CRect r, r2;
	CPoint p;

	GetCursorPos(&p);
	GetWindowRect(&r);
	GetClientRect(&r2);

	r.left += S_WIDTH;
	r.top += S_HEIGHT;
	r.right = r.left + r2.Width();
	r.bottom = r.top + r2.Height();

	if (r.PtInRect(p))
	{
		SetCursor(LoadCursor(NULL, MAKEINTRESOURCE(IDC_IBEAM)));
		return FALSE;
	}

	return CView::OnSetCursor(pWnd, nHitTest, message);
}

void CDSPView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	if (m_Form == 2 || (GetDocument()->IsPlaying && !GetDocument()->Paused)) return;
	if (point.x >= S_WIDTH && point.y >= S_HEIGHT)
	{
		point.x -= S_WIDTH;
		m_StartPos = m_Pos + (point.x << m_Scale);
		m_EndPos = m_StartPos;
		InvalidateRect(NULL, FALSE);
		m_Pressed = TRUE;
		SetCapture();
	}

	CView::OnLButtonDown(nFlags, point);
}

void CDSPView::OnLButtonUp(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	CRect r;
	GetClientRect(&r);
	if (!m_Pressed || m_Form == 2 || 
		(GetDocument()->IsPlaying && !GetDocument()->Paused)) return;
	if (point.x >= S_WIDTH && point.y >= S_HEIGHT &&
		point.x < r.Width() && point.y < r.Height())
	{
		point.x -= S_WIDTH;
		m_EndPos = m_Pos + (point.x << m_Scale);
		if (m_StartPos != m_EndPos)
		{
			InvalidateRect(NULL, FALSE);
			HideCaret();
		}
	}
	ReleaseCapture();
	m_Pressed = FALSE;

	CView::OnLButtonUp(nFlags, point);
}

void CDSPView::OnActivateView(BOOL bActivate, CView* pActivateView, CView* pDeactiveView) 
{
	// TODO: Add your specialized code here and/or call the base class
	if (bActivate)
		SetTimer(1, 20, NULL);
	else
		KillTimer(1);

	CView::OnActivateView(bActivate, pActivateView, pDeactiveView);
}

void CDSPView::OnMouseMove(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	if (m_Form == 2) return;
	if (m_Pressed)
	{
		CRect r;

		GetClientRect(&r);

		if (point.x >= S_WIDTH || nFlags == 0xFF)
		{
			point.x -= S_WIDTH;

			if (point.x > r.right) point.x = r.right;
			if (point.x < 0) point.x = r.left;

			m_EndPos = m_Pos + (point.x << m_Scale);
			if (nFlags != 0xFF) InvalidateRect(NULL, FALSE);
		}
	}

	CView::OnMouseMove(nFlags, point);
}


void CDSPView::OnTimer(UINT nIDEvent) 
{
	// TODO: Add your message handler code here and/or call default
	CDSPDoc *pDoc = GetDocument();
	MMTIME mtime;
	CRect r;
	GetWindowRect(&r);
	int n, m;
	static bool PrevPlay = false;
	static int PrevPos;

	mtime.wType = TIME_SAMPLES;

	if ((pDoc->IsPlaying && !pDoc->Paused && m_Form != 1) ||
		(!pDoc->IsPlaying && PrevPlay))
	{
		if (pDoc->IsPlaying)
		{
			waveOutGetPosition(pDoc->hwo, &mtime, sizeof(mtime));
			mtime.u.sample += pDoc->StartPos;
			m_Pos = (mtime.u.sample >> m_Scale) << m_Scale;
			n = ((r.Width() - S_WIDTH) >> 1) << m_Scale;
			m = m_PlayPos = m_Pos;
			m_Pos -= n;
			CheckPosition();
			m -= m_Pos;
		}
		else
			m_Pos = PrevPos;
		UpdatePosition();
		UpdateScrollParams();
		InvalidateRect(NULL, FALSE);
		PrevPlay = pDoc->IsPlaying;
		return;
	}

	if (m_Form == 2) return;
	if (m_Pressed)
	{
		CPoint p, point;

		GetCursorPos(&p);
		point.x = p.x - r.left - S_WIDTH;
		point.y = p.y - r.top - S_HEIGHT;
		GetClientRect(&r);

		if (point.x > r.right) 
		{
			OnHScroll(SB_LINERIGHT, 0, NULL);
			OnMouseMove(0xFF, point);
		}
		if (point.x < 0) 
		{
			OnHScroll(SB_LINELEFT, 0, NULL);
			OnMouseMove(0xFF, point);
		}
	}

	CView::OnTimer(nIDEvent);
}

void CDSPView::OnUpdateEditCopy(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	pCmdUI->Enable(m_StartPos != m_EndPos && !GetDocument()->IsPlaying && m_Form != 2);
}

int CDSPView::GetSelectionSize()
{
	return GetEndPos() - GetStartPos() + 1;
}

void CDSPView::GetSampleTime(int nSample, char *time)
{
	CDSPDoc *pDoc = GetDocument();
	int ms = int((double)nSample * 1000.0 / pDoc->h.w.nSamplesPerSec);
	GetTimeString(ms, time);
}

void CDSPView::GetTimeString(int msecs, char *time)
{
	int s = msecs / 1000;
	int m = msecs / 60000;
	int h = msecs / 3600000;
	sprintf(time, "%02d:%02d:%02d.%03d", h % 100, m % 60, s % 60, msecs % 1000);
}

int CDSPView::GetStartPos()
{
	return min(m_StartPos, m_EndPos);
}

int CDSPView::GetEndPos()
{
	return max(m_StartPos, m_EndPos);
}

int CDSPView::GetSampleSize()
{
	CDSPDoc *pDoc = GetDocument();
	return (pDoc->h.wBitsPerSample >> 3) * m_nChannels;
}

void CDSPView::OnUpdateEditCut(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	pCmdUI->Enable(m_StartPos != m_EndPos && !GetDocument()->IsPlaying &&
		GetSelectionSize() < m_nSamples && m_Form != 2);	
}

void CDSPView::OnUpdateEditClear(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	pCmdUI->Enable(m_StartPos != m_EndPos && !GetDocument()->IsPlaying &&
		GetSelectionSize() < m_nSamples && m_Form != 2);		
}

void CDSPView::GetClientRect(LPRECT lpRect)
{
	CView::GetClientRect(lpRect);
	lpRect->right -= S_WIDTH;
	lpRect->bottom -= S_HEIGHT;
}


void CDSPView::OnUpdateEditPaste(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	pCmdUI->Enable(!GetDocument()->IsPlaying && m_Form != 2);
}

void CDSPView::OnGotoend() 
{
	// TODO: Add your command handler code here
	m_Pos = m_nSamples - (1 << m_Scale);
	UpdatePosition();
	UpdateScrollParams();
	InvalidateRect(NULL, FALSE);
}

void CDSPView::OnGotostart() 
{
	// TODO: Add your command handler code here
	m_Pos = 0;	
	UpdatePosition();
	UpdateScrollParams();
	InvalidateRect(NULL, FALSE);
}

void CDSPView::OnForward() 
{
	// TODO: Add your command handler code here
	m_Pos = GetSampleForTime(GetSampleTime(m_StartPos) + 1001);
	m_StartPos = m_EndPos = m_Pos;
	UpdateScrollParams();
	InvalidateRect(NULL, FALSE);
}


void CDSPView::OnRewind() 
{
	// TODO: Add your command handler code here
	m_Pos = GetSampleForTime(GetSampleTime(m_StartPos) - 999);
	m_StartPos = m_EndPos = m_Pos;
	UpdateScrollParams();
	InvalidateRect(NULL, FALSE);
}

void CDSPView::OnUpdateClear(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	pCmdUI->Enable(m_StartPos != m_EndPos && !GetDocument()->IsPlaying && m_Form != 2);
}

void CDSPView::OnSelectall() 
{
	// TODO: Add your command handler code here
	m_StartPos = 0;
	m_EndPos = m_nSamples - 1;
	InvalidateRect(NULL, FALSE);
}

void CDSPView::OnSelfragment() 
{
	// TODO: Add your command handler code here
	if (seldlg.DoModal() != IDOK) return;
	
	m_StartPos = GetSampleForTime(seldlg.m_Hour1 * 3600000 +
		seldlg.m_Min1 * 60000 + seldlg.m_Sec1 * 1000 + seldlg.m_Msec1);

	m_EndPos = GetSampleForTime(seldlg.m_Hour2 * 3600000 +
		seldlg.m_Min2 * 60000 + seldlg.m_Sec2 * 1000 + seldlg.m_Msec2);

	InvalidateRect(NULL, FALSE);
}

