// RecordDlg.cpp : implementation file
//

#include "stdafx.h"
#include "dsp.h"
#include "RecordDlg.h"
#include "filters.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CRecordDlg dialog

CRecordDlg *pDlg;

CRecordDlg::CRecordDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CRecordDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CRecordDlg)
	m_SmpRate = 2;
	m_SmpSize = 0;
	m_Channels = 0;
	//}}AFX_DATA_INIT
	IsPlaying = false;
	IsRecording = false;
	Paused = false;
	SetFormatData(wf2);
	memcpy(&wf1, &wf2, sizeof(WAVEFORMATEX));
	block = new char[BLOCK_SIZE << 2];
	old_pixels = new unsigned char[512];
	memset(block, 0x80, BLOCK_SIZE << 2);
	memset(old_pixels, 0, 512);
	hwo = 0;
	hwi = 0;
	beg = end = NULL;
	pDlg = this;
}

CRecordDlg::~CRecordDlg()
{
	delete block;
}

void CRecordDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CRecordDlg)
	DDX_Control(pDX, IDOK, m_Ok);
	DDX_Control(pDX, IDC_PAUSE, m_Pause);
	DDX_Control(pDX, IDC_PLAY, m_Play);
	DDX_Control(pDX, IDC_STOP, m_Stop);
	DDX_Control(pDX, IDC_REC, m_Rec);
	DDX_Radio(pDX, IDC_SMPRATE, m_SmpRate);
	DDX_Radio(pDX, IDC_SMPSIZE, m_SmpSize);
	DDX_Radio(pDX, IDC_CHANNEL, m_Channels);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CRecordDlg, CDialog)
	//{{AFX_MSG_MAP(CRecordDlg)
	ON_BN_CLICKED(IDC_PLAY, OnPlay)
	ON_BN_CLICKED(IDC_PAUSE, OnPause)
	ON_BN_CLICKED(IDC_REC, OnRec)
	ON_BN_CLICKED(IDC_STOP, OnStop)
	ON_BN_CLICKED(IDC_SMPRATE, OnSmprate)
	ON_BN_CLICKED(IDC_SMPSIZE, OnSmpsize)
	ON_BN_CLICKED(IDC_CHANNEL, OnChannel)
	ON_WM_PAINT()
	ON_BN_CLICKED(IDC_RADIO3, OnSmprate)
	ON_BN_CLICKED(IDC_RADIO5, OnSmprate)
	ON_BN_CLICKED(IDC_RADIO6, OnSmprate)
	ON_BN_CLICKED(IDC_RADIO7, OnSmpsize)
	ON_BN_CLICKED(IDC_RADIO8, OnChannel)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CRecordDlg message handlers

void CRecordDlg::OnPlay() 
{
	// TODO: Add your control notification handler code here
	if (Paused) 
	{
		waveOutRestart(hwo);
		Paused = false;
	}
	if (IsRecording || IsPlaying || !beg) return;

	MMRESULT n;

	cur = beg;
	
	n = waveOutOpen(&hwo, 0, &wf1, (DWORD)waveOutProc1, (DWORD)AfxGetInstanceHandle(), CALLBACK_FUNCTION);
	if (n != MMSYSERR_NOERROR)
	{
		char str[20];
		itoa(n, str, 16);
		MessageBox("   waveOutOpen", str, MB_OK | MB_ICONSTOP);
		return;
	}

	WriteBuffer();
	WriteBuffer();
	IsPlaying = true;
}

void CRecordDlg::OnPause() 
{
	// TODO: Add your control notification handler code here
	if (IsRecording || Paused) return;
	Paused = true;
	waveOutPause(hwo);
}

void CRecordDlg::OnRec() 
{
	// TODO: Add your control notification handler code here
	if (IsPlaying || IsRecording) return;
	m_Ok.EnableWindow();
	
	MMRESULT n;
	
	DeleteList();
	memcpy(&wf1, &wf2, sizeof(WAVEFORMATEX));

	n = waveInOpen(&hwi, 0, &wf1, (DWORD)waveInProc, (DWORD)AfxGetInstanceHandle(), CALLBACK_FUNCTION);
	if (n != MMSYSERR_NOERROR)
	{
		MessageBox("   waveInOpen", NULL, MB_OK | MB_ICONSTOP);
		return;
	}

	AddBuffer();
	AddBuffer();
	waveInStart(hwi);
	IsRecording = true;
}

void CRecordDlg::OnStop() 
{
	// TODO: Add your control notification handler code here
	if (!IsPlaying && !IsRecording) return;
	if (IsPlaying)
	{
		IsPlaying = false;
		Paused = false;
		waveOutReset(hwo);
		waveOutClose(hwo);
		hwo = 0;
	}
	if (IsRecording)
	{
		IsRecording = false;
		waveInStop(hwi);
		waveInReset(hwi);
		waveInClose(hwi);
		hwi = 0;
	}
}

void CRecordDlg::OnSmprate() 
{
	// TODO: Add your control notification handler code here
	UpdateData();
	SetFormatData(wf2);
}

void CRecordDlg::OnSmpsize() 
{
	// TODO: Add your control notification handler code here
	UpdateData();
	SetFormatData(wf2);
}

void CRecordDlg::OnChannel() 
{
	// TODO: Add your control notification handler code here
	UpdateData();
	SetFormatData(wf2);
}

void CRecordDlg::SetFormatData(WAVEFORMATEX& wf)
{
	int smp_rate;

	smp_rate = GetSampleRate(m_SmpRate);
	wf.wFormatTag = WAVE_FORMAT_PCM;
	wf.nSamplesPerSec = smp_rate;
	wf.nChannels = m_Channels + 1;
	wf.wBitsPerSample = 1 << (m_SmpSize + 3);
	wf.nBlockAlign = wf.nChannels << m_SmpSize;
	wf.nAvgBytesPerSec = smp_rate * wf.nBlockAlign;
	wf.cbSize = 0;
}

void CRecordDlg::DrawBlockData()
{
	CRect r;
	int s, i = 0, index, offs;

	CWnd *w = GetDlgItem(IDC_BLOCKGRAPH);
	CDC *dc = w->GetDC();
	w->GetClientRect(&r);
	dc->SelectStockObject(WHITE_PEN);
	offs = r.Height() / 2;

	for (; i < r.Width(); i++)
	{
		dc->SetPixel(CPoint(i, old_pixels[i]), 0);
		index = i << (wf1.nChannels - 1);
		if (wf1.wBitsPerSample == 16)
		{
			s = (unsigned short)(((unsigned short *)block)[index] - 32768);
			s >>= 8;
		}
		else
			s = (unsigned char)block[index];
		s = 64 - (s >> 1) + offs;
		old_pixels[i] = s;
		dc->SetPixel(CPoint(i, s), 0x00FFFFFF);
	}

	ReleaseDC(dc);
}

void CRecordDlg::OnPaint()
{
	CPaintDC dc(this);
	CRect r;
	CWnd *w = GetDlgItem(IDC_BLOCKGRAPH);
	CDC *dc2 = w->GetDC();

	w->GetClientRect(&r);
	dc2->SelectStockObject(BLACK_PEN);
	dc2->SelectStockObject(BLACK_BRUSH);
	dc2->Rectangle(&r);
	dc2->SelectStockObject(WHITE_PEN);
	ReleaseDC(dc2);

	DrawBlockData();
}

void CRecordDlg::AddBuffer()
{
	WAVEHDR *wh = new WAVEHDR;
	int size = wf1.nBlockAlign * BLOCK_SIZE;
	memset(wh, 0, sizeof(WAVEHDR));
	wh->lpData = new char[size];
	wh->dwBufferLength = size;
	waveInPrepareHeader(hwi, wh, sizeof(WAVEHDR));
	waveInAddBuffer(hwi, wh, sizeof(WAVEHDR));
}

void CRecordDlg::WriteBuffer()
{
	WAVEHDR *wh = new WAVEHDR;
	memset(wh, 0, sizeof(WAVEHDR));
	wh->lpData = new char[cur->size];
	memcpy(wh->lpData, cur->data, cur->size);
	wh->dwBufferLength = cur->size;
	cur = cur->next;
	waveOutPrepareHeader(hwo, wh, sizeof(WAVEHDR));
	waveOutWrite(hwo, wh, sizeof(WAVEHDR));
}

void CALLBACK waveOutProc1(HWAVEOUT hwo, UINT uMsg, DWORD dwInstance,
						   DWORD dwParam1, DWORD dwParam2)
{
	if (uMsg != WOM_DONE) return;
	WAVEHDR *wh = (WAVEHDR *)dwParam1;
	waveOutUnprepareHeader(pDlg->hwo, wh, sizeof(WAVEHDR));
	BlockList *b = pDlg->cur;
	if (b == NULL)
	{
		pDlg->OnStop();
		return;
	}
	if (pDlg->IsPlaying) pDlg->WriteBuffer();
	memcpy(pDlg->block, wh->lpData, wh->dwBufferLength);
	delete wh->lpData;
	delete wh;
	pDlg->DrawBlockData();
}

void CALLBACK waveInProc(HWAVEOUT hwo, UINT uMsg, DWORD dwInstance,
						 DWORD dwParam1, DWORD dwParam2)
{
	if (uMsg != WIM_DATA) return;
	WAVEHDR *wh = (WAVEHDR *)dwParam1;
	if (pDlg->IsRecording) pDlg->AddBuffer();
	int n = wh->dwBytesRecorded;
	if (!n) return;
	memcpy(pDlg->block, wh->lpData, n);
	waveInUnprepareHeader(pDlg->hwi, wh, sizeof(WAVEHDR));
	delete wh->lpData;
	delete wh;
	pDlg->AppendList(n);
	pDlg->DrawBlockData();
}

void CRecordDlg::OnOK() 
{
	// TODO: Add extra validation here
	OnStop();

	data = new char[m_Blocks * BLOCK_SIZE * wf1.nBlockAlign];
	BlockList *b = beg;
	int pos = 0;

	while (b != NULL)
	{
		beg = beg->next;
		memcpy(data + pos, b->data, b->size);
		pos += b->size;
		delete b->data;
		delete b;
		b = beg;
	}
	datasize = pos;

	CDialog::OnOK();
}

void CRecordDlg::OnCancel() 
{
	// TODO: Add extra cleanup here
	OnStop();	
	DeleteList();
	CDialog::OnCancel();
}

void CRecordDlg::AppendList(int size)
{
	BlockList *b = new BlockList;

	b->data = new char[size];
	memcpy(b->data, block, size);
	b->size = size;
	b->next = NULL;

	if (beg == NULL)
		beg = b;
	else
		end->next = b;
	end = b;

	m_Blocks++;
}

void CRecordDlg::DeleteList()
{
	BlockList *b = beg;

	while (b != NULL)
	{
		beg = beg->next;
		delete b->data;
		delete b;
		b = beg;
	}

	beg = NULL;
	end = NULL;
	m_Blocks = 0;
}


BOOL CRecordDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	// TODO: Add extra initialization here
	m_Ok.EnableWindow(false);
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}
