// UserfilterDlg.cpp : implementation file
//

#include "stdafx.h"
#include "dsp.h"
#include "UserfilterDlg.h"
#include "filters.h"
#include <fstream.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CUserfilterDlg dialog


CUserfilterDlg::CUserfilterDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CUserfilterDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CUserfilterDlg)
	m_WindowType = 0;
	m_ApplyTo = -1;
	m_KernelSize = 512;
	//}}AFX_DATA_INIT
	memset(m_Bands, 0, sizeof(m_Bands));
}


void CUserfilterDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CUserfilterDlg)
	DDX_Radio(pDX, IDC_WINDOWTYPE, m_WindowType);
	DDX_CBIndex(pDX, IDC_APPLYTO, m_ApplyTo);
	DDX_Text(pDX, IDC_KERNELSIZE, m_KernelSize);
	DDV_MinMaxInt(pDX, m_KernelSize, 16, 1024);
	for (int i = 0; i < 10; i++)
	{
		int n = 60 - m_Bands[i];
		DDX_Slider(pDX, IDC_SLIDER1 + i, n);
		m_Bands[i] = 60 - n;
	}
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CUserfilterDlg, CDialog)
	//{{AFX_MSG_MAP(CUserfilterDlg)
	ON_WM_VSCROLL()
	ON_BN_CLICKED(IDC_DEFAULT, OnDefault)
	ON_BN_CLICKED(IDC_SAVE, OnSave)
	ON_BN_CLICKED(IDC_LOAD, OnLoad)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CUserfilterDlg message handlers

BOOL CUserfilterDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();
	int n, i = 0;
	char s[10];
	
	// TODO: Add extra initialization here
	for (; i < 10; i++)
	{
		CSliderCtrl *sl = (CSliderCtrl *)GetDlgItem(IDC_SLIDER1 + i);
		sl->SetRangeMin(0);
		sl->SetRangeMax(120);
		sl->SetPos(60 - m_Bands[i]);
		sl->SetTicFreq(6);
		n = GetEqualizerFreq(i, m_HalfRate);
		m_Freqs[i] = n;
		if (n < 1000)
			sprintf(s, "%d", n);
		else
			if (n % 1000)
				sprintf(s, "%d.%d", n / 1000, (n - n / 1000) % 10);
			else
				sprintf(s, "%d", n / 1000);
		SetDlgItemText(IDC_FREQ1 + i, s);
	}
	
	if (!m_ApplyTo)
	{
		CComboBox *cb = (CComboBox *)GetDlgItem(IDC_APPLYTO);
		cb->DeleteString(1);
	}
	
	SetDBText(0, 0, true);

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CUserfilterDlg::OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar)
{
	// TODO: Add your message handler code here and/or call default
	CSliderCtrl *sl = (CSliderCtrl *)pScrollBar;
	int n = sl->GetPos(), i;

	i = sl->GetDlgCtrlID() - IDC_SLIDER1;
	n = 60 - n;
	SetDBText(i, n, false);
	
	CDialog::OnVScroll(nSBCode, nPos, pScrollBar);
}

void CUserfilterDlg::SetDBText(int n, int val, bool all)
{
	char s[10];

	for (int i = all ? 0 : n; i < 10; i++)
	{
		int n = all ? m_Bands[i] : val;
		if (n > 0) 
			sprintf(s, "+%d", n);
		else
			if (n < 0) sprintf(s, "%d", n);
		else
			strcpy(s, "0");
		SetDlgItemText(IDC_FREQ11 + i, s);
		if (!all) return;
	}
}


void CUserfilterDlg::OnDefault() 
{
	// TODO: Add your control notification handler code here
	memset(m_Bands, 0, sizeof(m_Bands));
	m_KernelSize = 512;
	m_WindowType = 0;
	UpdateData(false);
	SetDBText(0, 0, true);
}

void CUserfilterDlg::OnSave() 
{
	// TODO: Add your control notification handler code here
	UserFilter uf;

	CFileDialog s(false, "fil", NULL, OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,
		"   (*.fil)|*.fil|  (*.*)|*.*||",
		this);

	if (s.DoModal() != IDOK) return;

	ofstream out(s.GetPathName(), ios::binary);
	if (!out)
	{
		MessageBox("  ", "", MB_OK | MB_ICONSTOP);
		return;
	}

	UpdateData(true);
	memcpy(uf.Bands, m_Bands, sizeof(m_Bands));
	uf.WindowType = m_WindowType;
	uf.KernelSize = m_KernelSize;
	out.write((char *)&uf, sizeof(uf));
	out.close();
}

void CUserfilterDlg::OnLoad() 
{
	// TODO: Add your control notification handler code here
	UserFilter uf;
	int i = 0;

	CFileDialog o(true, "fil", NULL, OFN_HIDEREADONLY,
		"   (*.fil)|*.fil|  (*.*)|*.*||",
		this);

	if (o.DoModal() != IDOK) return;

	ifstream in(o.GetPathName(), ios::binary);
	if (!in)
	{
		MessageBox("  ", "", MB_OK | MB_ICONSTOP);
		return;
	}

	in.read((char *)&uf, sizeof(uf));
	for (; i < 10; i++)
		if (uf.Bands[i] < -60 || uf.Bands[i] > 60) break;
	if (i != 10 || uf.KernelSize < 16 || uf.KernelSize > 1024 ||
		uf.WindowType > 1 || uf.WindowType < 0)
	{
		MessageBox("  ", "", MB_OK | MB_ICONSTOP);
		return;
	}
	in.close();

	memcpy(m_Bands, uf.Bands, sizeof(m_Bands));
	m_WindowType = uf.WindowType;
	m_KernelSize = uf.KernelSize;

	UpdateData(false);
	SetDBText(0, 0, true);
}

