#ifndef _VECTOR_H_
#define _VECTOR_H_
#include <math.h>

class Vector {
  public:
    double x, y;
    Vector() { }
    Vector(double x, double y)
    {
      Vector::x = x;
      Vector::y = y;
    }
    Vector(Vector& v)
    {
      x = v.x;
      y = v.y;
    }
    Vector(double v)
    {
      x = y = v;
    }
    Vector& operator =(Vector& v)
    {
      x = v.x;
      y = v.y;
      return *this;
    }
    Vector& operator =(double d)
    {
      x = y = d;
      return *this;
    }
    Vector& operator +=(Vector& v)
    {
      x += v.x;
      y += v.y;
      return *this;
    }
    Vector& operator -=(Vector& v)
    {
      x -= v.x;
      y -= v.y;
      return *this;
    }
    Vector operator -()
    {
      return Vector(-x, -y);
    }
    Vector& operator *=(Vector& v)
    {
      x *= v.x;
      y *= v.y;
      return *this;
    }
    Vector& operator *=(double d)
    {
      x *= d;
      y *= d;
      return *this;
    }
    Vector& operator /=(Vector& v)
    {
      x /= v.x;
      y /= v.y;
      return *this;
    }
    Vector& operator /=(double d)
    {
      x /= d;
      y /= d;
      return *this;
    }
    friend bool operator !=(Vector& v1, Vector& v2)
    {
      return (v1.x != v2.x || v1.y != v2.y);
    }
    friend bool operator ==(Vector& v1, Vector& v2)
    {
      return (v1.x == v2.x && v1.y == v2.y);
    }
    friend Vector operator +(Vector& v1, Vector& v2)
    {
      return Vector(v1.x + v2.x, v1.y + v2.y);
    }
    friend Vector operator -(Vector& v1, Vector& v2)
    {
      return Vector(v1.x - v2.x, v1.y - v2.y);
    }
    friend Vector operator *(Vector& u, Vector& v)
    {
      return Vector(u.x * v.x, u.y * v.y);
    }
    friend Vector operator *(Vector& v, double d)
    {
      return Vector(v.x * d, v.y * d);
    }
    friend Vector operator *(double d, Vector& v)
    {
      return Vector(v.x * d, v.y * d);
    }
    friend Vector operator /(Vector& u, Vector& v)
    {
      return Vector(u.x / v.x, u.y / v.y);
    }
    friend Vector operator /(Vector& v, double d)
    {
      return Vector(v.x / d, v.y / d);
    }
    double& operator [](int n)
    {
      return *(&x + n);
    }
    double operator !()
    {
      return sqrt(x * x + y * y);
    }
    friend double operator &(Vector& u, Vector& v)
    {
      return u.x * v.x + u.y * v.y;
    }
};

#endif